// #include "csvFunctions .h"
#include <Arduino.h>
#include <SPIFFS.h>
#include <Preferences.h>
#include <FS.h>
#include <vector>
#include <string>
using namespace std;

/*
1) Remember create a folder titled 'data' within the project. 
2) Then add the file you want to read from to the data folder
3) Then click the projectIO icon and select the board name
4) Underplatform select 'Build Filesystem Image'
5) Next select 'Upload Filesystem Image'
6) Now you are able to run the script and successfully read from the file included in the data folder
*/


vector<string> column1;
vector<string> column3;


// Function to read the CSV file and print columns 1 and 3 to the Serial monitor
void readCSVFile() {
  // Open the CSV file
  File file = SPIFFS.open("/data/horn240427.csv", "r");
  
  if (!file) {
    Serial.println("Failed to open file for reading");
    return;
  } else {
    Serial.println("Successful File Read");
  }

  // Read each line of the file
  while (file.available() != 0) {
   String line = file.readStringUntil('\n');
    int firstCommaIdx = line.indexOf(',');
    int secondCommaIdx = line.indexOf(',', firstCommaIdx + 1);
    int thirdCommaIdx = line.indexOf(',', secondCommaIdx + 1);

    if (firstCommaIdx != -1 && secondCommaIdx != -1) {
      String col1 = line.substring(0, firstCommaIdx);
      String col3 = line.substring(secondCommaIdx + 1, thirdCommaIdx);

      column1.push_back(col1.c_str());
      column3.push_back(col3.c_str());
    }

  }

  // Close the file
  file.close();

  Serial.println("Column 1 Values:");
  for (const auto& value : column1) {
    Serial.println(value.c_str());
  }

  Serial.println("Column 3 Values:");
  for (const auto& value : column3) {
    Serial.println(value.c_str());
  }

}

void printCSV() {
  delay(5000);
  Serial.begin(115200);
  Serial.println("Start Test");
  
  // Initialize SPIFFS
  if (!SPIFFS.begin(true)) {
    Serial.println("Failed to mount file system");
    return;
  } else {
    Serial.println("Successful file system mount");
  }

  // Call function to read and print data from CSV file
  readCSVFile();
          
}

// Function to write data to a CSV file
void writeArrayToCSV(const vector<vector<double>>& array, const char* filename) {
  // Open the file for writing
  File file = SPIFFS.open("/data"+String(filename), FILE_WRITE);
  
  if (!file) {
    Serial.println("Failed to open file for writing");
    return;
  }

  // Iterate over each row of the array and write to the file
  for (const auto& row : array) {
    for (size_t i = 0; i < row.size(); ++i) {
      file.print(row[i]);
      if (i != row.size() - 1) {
        file.print(",");
      }
    }
    file.println();
  }

  // Close the file
  file.close();
}

// Example 
void writeToCSVExample() {
  // Example array
  vector<vector<double>> array = {
      {1.6, 2, 3},
      {4, 5, 6},
      {7, 8, 9}
  };

  // Initialize SPIFFS
  if (!SPIFFS.begin(true)) {
    Serial.println("Failed to mount file system");
    return;
  }

  // Call function to write array to CSV file
  writeArrayToCSV(array, "/parseTest.csv");

  Serial.println("CSV file written successfully");
}

void setup() {
  Serial.begin(115200);
  delay(1000);
  writeToCSVExample();
  printCSV();
  
}

void loop() {
  // Empty loop
}


/*#include <Arduino.h>
#include <stdio.h>
#include <random>

const char startData = '*';
const char separatorData = ',';
const char endData = '#';
float data1, data2, data3, data4, data5;
String dataPackage;

double powerDetect(float theta){
    Serial.println("powerDetect");
    //////  -- ADC Code --  //////
    // start detecting DC
    pinMode(1, INPUT); 
    analogReadResolution(12);

    // initialize variables
    double detectorValue = 0;
    double detectorVolts = 0; 
    double detectorValueTot = 0;
    double detectorVoltsTot = 0;

    // 10 measurement hold to read out from GPIO 1 (pin39, ADC1_CH0) 
    for (int i = 0; i<10; i++){
        // get detector values
        detectorValue = analogRead(1);
        detectorVolts = analogReadMilliVolts(1);

        // print current detector values (DEV)
        //Serial.print("Raw Value = ");
        //Serial.println(detectorValue);
        //Serial.print("mV Value = ");
        //Serial.println(detectorVolts);
    
        // add to running total for this iteration
        detectorValueTot += detectorValue;
        detectorVoltsTot += detectorVolts;
    
        // delay for clean reads
        // needs to be vTaskDelay so that we go to the Power_Monitor task 
        vTaskDelay(100); 
    }

    // calculate average
    double detectorVoltsAvg = detectorVoltsTot / 10;
    //dataPackage = (String) startData + theta + separatorData + detectorVoltsAvg + endData;
    Serial.print("Phase ");
    Serial.print(theta);
    Serial.print(" Voltage ");
    Serial.println(detectorVoltsAvg);

    delay(1000);

    return detectorVoltsAvg;
}

void setup(){
    Serial.begin(112500);
    delay(1000);

    for (int j = 0; j <= 180; j++){
        powerDetect(j);
    }

}

void loop(){
    // powerDetect(0);

    /*data1 = 250.555555;
    data2 = 270.666666;
    data3 = 290.777777;
    data4 = 310.888888;
    data5 = 330.999999;

    dataPackage = (String) startData + data1 + separatorData 
                                        + data2 + separatorData
                                        + data3 + separatorData
                                        + data4 + separatorData
                                        + data5 + endData;

    delay(100);

    Serial.println(dataPackage);


}*/