#include <Arduino.h>
#include <stdio.h>
#include "motorFunctions.h"
#include "antennaFunctions.h"
#include "spiFunctions.h"
#include "usbFunctions.h"
#include <SPIFFS.h>
#include <Preferences.h>
#include <FS.h>
#include <string>

#include "esp_rom_gpio.h"
#include "driver/gpio.h"

#include "Wire.h"
#include <vector>


#define MotorA1 7
#define MotorA2 16
#define MotorB2 15
#define MotorB1 17

#define SDA_PIN 4 //4  // Change to your SDA pin
#define SCL_PIN 5 //5 // Change to your SCL pin

volatile float theta = 0; // select iteration

////// -- FreeRTOS Definitions -- ///////
TaskHandle_t Power_Monitor;
TaskHandle_t Main_State_Machine;

////// -- State Machine Setup -- //////
typedef enum state_machine_s {
    STARTUP, // start here
    MOTOR_DEPLOY,
    USB_COMMS,
    ANTENNA_OP, 
    IDLE,
    LOW_POWER,
    TEST
} state_machine_t;

// initialize the current state
volatile state_machine_t currentState = STARTUP;

//////  -- Task Code: Power Monitor --  //////
void Power_Monitor_Fxn( void * pvParameters ){
  Serial.println();
  Serial.print("Power_Monitor running on core ");
  Serial.print(xPortGetCoreID());
  //Serial.print(". currentState = ");
  //Serial.println(currentState);

  for(;;){
    //digitalWrite(led1, HIGH);
    Serial.println();
    Serial.print("Power_Monitor running on core ");
    Serial.print(xPortGetCoreID());
    //Serial.print(". currentState = ");
    //Serial.println(currentState);
    
    Serial.println();
    Serial.println("currentState = USB_COMMS");

    // loop I2C readout
    float comp = readComp(); // Reads  value from I2C connection
    Serial.printf("I2C Communication: %.1f \n", comp);
    Serial.println();
    delay(5000); // Delays 5s
  } 
}

void Main_State_Machine_Fxn( void * pvParameters ){
  Serial.println();
  Serial.print("Main_State_Machine running on core ");
  Serial.print(xPortGetCoreID());
  //Serial.print(". currentState = ");
  //Serial.println(currentState);

  for(;;){
    // original dual core processing code
    //digitalWrite(led2, HIGH);
    Serial.println();
    Serial.println("Loop running on core ");
    Serial.print(xPortGetCoreID());
    //Serial.print(". currentState = ");
    //Serial.println(currentState);
 
  switch(currentState){
    case STARTUP:
      Serial.println("currentState = STARTUP");

      currentState = MOTOR_DEPLOY;
      break;

    case MOTOR_DEPLOY:
      Serial.println("currentState = MOTOR_DEPLOY");
      
      /*float lambda = setLambda();
      motorSetup();
      moveMotor(lambda);
      Serial.println("Motor deployement finished");*/

      currentState = USB_COMMS;
      break;  
    case USB_COMMS:
      Serial.println("currentState = USB_COMMS");

      // elected to run this on other core
      // float powerConsumed  = readComp(); // Reads value from INA236A
      // Serial.printf("Power Raw Readout: %.1f \n", powerConsumed);
      // delay(5000); // Delays 5 seconds*/
  
      currentState = ANTENNA_OP;
      break;

    case ANTENNA_OP:
      Serial.println();
      Serial.println("currentState = ANTENNA_OP");

      // setWeights(theta); // move away from this for demo day contingency plan
        double powerDetectOutput;
        
        // step through phase values, for power combiner contingency plan
        for (int j = 0; j <= 180; j++){
            powerDetectOutput = powerDetect(j);
        }

        // done from Matlab script on demo day
        // writeArrayCSV(powerDetectOutput, "/PDOutputVolts.csv");
        // Serial.println("CSV file written successfully");

        currentState = STARTUP;
        break;

    case IDLE:{
        Serial.println("currentState = IDLE");

        currentState = USB_COMMS;
        break;
      }
      
    case TEST: {
        Serial.println("currentState = TEST");
        
        // used strictly in development

        break;
      }
    }
  } 
} 

/*
void configureGPIO() {
    // Configure GPIO pin 19 as an output
    gpio_config(MotorA1, );
    gpio_pad_select_gpio(MotorA1);
    //gpio_set_direction(32, GPIO_MODE_OUTPUT);

    gpio_pad_select_gpio(MotorA2);
    //gpio_set_direction(MotorA2, GPIO_MODE_OUTPUT);
}
*/

void setup() {
  Serial.begin(115200);
  delay(4000);

  Wire.begin(SDA_PIN, SCL_PIN);

  // deploy motors
  Serial.println("Begin WCUBE Project Sequence");
  float lambda = setLambda();
  motorSetup();
  moveMotor(lambda);
  Serial.println("Motor deployement finished");

  // start up I2C
  delay(5000);
  Serial.print("Startup I2C");
  int config = configReg(); // Reads value in configuration register
  Serial.begin(115200);
  Serial.printf("I2C Configuration Register: 0x%x\n", config);

  // start up SPIFFS for SPI
  delay(5000);
  if (!SPIFFS.begin()) {
    Serial.println("An Error has occurred while mounting SPIFFS");
    return;
  }

  //gpio_config(MotorA1, );
  //pinMode(MotorA1, OUTPUT); // Motor A1
  //pinMode(MotorA2, OUTPUT); // Motor A2
  //pinMode(MotorB2, OUTPUT); // Motor B2
  //pinMode(MotorB1, OUTPUT); // Motor B1

  //////  -- Task Setup --  //////
  xTaskCreatePinnedToCore(
      Power_Monitor_Fxn,   /* Task function. */
      "Power_Monitor",     /* name of task. */
      10000,       /* Stack size of task */
      NULL,        /* parameter of the task */
      1,           /* priority of the task */
      &Power_Monitor,      /* Task handle to keep track of created task */
      0);          /* pin task to core 0 */                  
  delay(500); 

  xTaskCreatePinnedToCore(
      Main_State_Machine_Fxn,   /* Task function. */
      "Main_State_Machine",     /* name of task. */
      10000,       /* Stack size of task */
      NULL,        /* parameter of the task */
      1,           /* priority of the task */
      &Main_State_Machine,      /* Task handle to keep track of created task */
      1);          /* pin task to core 0 */                  
  delay(500);
}

void loop() {
}